Package: rebmix
Version: 2.17.1
Authors@R: c(person("Marko", "Nagode", role = c("aut", "cre"),
                     email = "marko.nagode@fs.uni-lj.si",
                     comment = c(ORCID = "0000-0003-0637-3812")),
             person("Branislav", "Panic", role = "ctb",
                     comment = c(ORCID = "0000-0001-8349-8550")),
             person("Jernej", "Klemenc", role = "ctb",
                     comment = c(ORCID = "0000-0002-6778-6728")),
             person("Simon", "Oman", role = "ctb",
                     comment = c(ORCID = "0000-0001-8213-0818")))
Title: Finite Mixture Modeling, Clustering & Classification
Description: Random univariate and multivariate finite mixture model generation, estimation, clustering, latent class analysis and classification. Variables can be continuous, discrete, independent or dependent and may follow normal, lognormal, Weibull, gamma, Gumbel, binomial, Poisson, Dirac, uniform or circular von Mises parametric families.
Depends: R (>= 3.1.0)
Imports: methods, stats, utils, graphics, grDevices
License: GPL (>= 2)
Author: Marko Nagode [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-0637-3812>),
  Branislav Panic [ctb] (ORCID: <https://orcid.org/0000-0001-8349-8550>),
  Jernej Klemenc [ctb] (ORCID: <https://orcid.org/0000-0002-6778-6728>),
  Simon Oman [ctb] (ORCID: <https://orcid.org/0000-0001-8213-0818>)
Maintainer: Marko Nagode <marko.nagode@fs.uni-lj.si>
NeedsCompilation: yes
Packaged: 2025-11-10 11:25:30 UTC; PCNagodeM
Repository: CRAN
Date/Publication: 2025-11-10 13:10:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 21:44:03 UTC; unix
Archs: rebmix.so.dSYM
