useDynLib(rebmix, .registration = TRUE, .fixes = "C_")

import("methods", "stats", "utils", "graphics", "grDevices")

export("kseq")

exportClasses("RNGMIX.Theta",
              "RNGMVNORM.Theta",
### Panic Branislav.
              "EMMIX.Theta",
              "EMMVNORM.Theta",
### End              
              "RNGMIX",
              "RNGMVNORM",
              "REBMIX",
              "REBMVNORM",
              "REBMIX.boot",
              "REBMVNORM.boot",
              "RCLRMIX",
              "RCLRMVNORM",
              "RCLSMIX",
              "RCLSMVNORM",
              "RCLS.chunk",
### Panic Branislav.
              "EM.Control")
### End

exportMethods("RNGMIX",
              "REBMIX",
              "RCLRMIX",
              "RCLSMIX",
### Panic Branislav.
              "EMMIX",
### End            
              "logL",
              "AIC",
              "AIC3",
              "AIC4",
              "AICc",
              "CAIC",
              "BIC",
              "HQC",
              "MDL2",
              "MDL5",
              "AWE",
              "CLC",
              "ICL",
              "ICLBIC",
              "PRD",
              "SSE",
              "PC",
              "show",
              "summary",
              "plot",
              "boot",
              "demix",
              "pemix",
              "dfmix",
              "pfmix",
              "split",
              "chunk",
              "BFSMIX",
              "a.d",
              "a.theta1<-",
              "a.theta2<-",
              "a.theta3<-",
              "a.Dataset.name",
              "a.rseed",
              "a.n",
              "a.Theta",
              "a.Dataset",
              "a.Zt",
              "a.w",
              "a.w<-",
              "a.Variables",
              "a.ymin",
              "a.ymax",
              "a.Preprocessing",
              "a.cmax",
              "a.Criterion",
              "a.pdf",
              "a.theta1",
              "a.theta2",
              "a.theta3",
              "a.theta1.all",
              "a.theta2.all",
              "a.theta3.all",
              "a.theta1.all<-",
              "a.theta2.all<-",
              "a.theta3.all<-",              
              "a.K",
              "a.K<-",              
              "a.y0",
              "a.ar",
              "a.Restraints",
              "a.summary",
### Panic Branislav.
              "a.summary.EM",
### End
              "a.pos",
              "a.opt.c",
              "a.opt.IC",
              "a.opt.logL",
              "a.opt.D",
              "a.all.K",
              "a.all.IC",
              "a.Bootstrap",
              "a.B",
              "a.replace",
              "a.prob",
              "a.c",
              "a.c.se",
              "a.c.cv",
              "a.c.mode",
              "a.c.prob",
              "a.w.se",
              "a.w.cv",
              "a.Theta.se",
              "a.Theta.cv",
              "a.Zp",
              "a.p",
              "a.pi",
              "a.P",
              "a.tau",
              "a.from",
              "a.to",
              "a.EN",
              "a.ED",
### Panic Branislav.
              "a.Rule",
### End              
              "a.o",
              "a.s",
              "a.ntrain",
              "a.ntest",
              "a.CM",
              "a.Accuracy",
              "a.Error",
              "a.Precision",
              "a.Sensitivity",
              "a.Specificity",
              "a.Chunks",
              "a.levels",
              "a.train",
              "a.Zr",
              "a.test",
### Panic Branislav.
              "a.strategy",
              "a.strategy<-",
              "a.variant",
              "a.variant<-",
              "a.acceleration",
              "a.acceleration<-",
              "a.tolerance",
              "a.tolerance<-",
              "a.acceleration.multiplier",
              "a.acceleration.multiplier<-",
              "a.maximum.iterations",
              "a.maximum.iterations<-",
              "a.eliminate.zero.components",
              "a.eliminate.zero.components<-",
              "a.likelihood.estimation.rule",
              "a.likelihood.estimation.rule<-",
              "a.likelihood.tolerance.check",
              "a.likelihood.tolerance.check<-",
              "optbins",
              "bins",
### End 
              "a.Y",
              "a.Y<-",
              "a.h",
              "a.ns",
              "fhistogram",
              "chistogram",
              "mapclusters",
              "labelmoments",
              "mergelabels")
